#include <windows.h>
#include <GL/glut.h>
#include <stdio.h>
#include <assert.h>
#include "rawLoader.h"

#pragma pack(1)

// the TGA image header ( == 18 bytes in size)
struct Header
{
	// do not care about this data
	unsigned char Rubbish[12];

	// the image width....
	unsigned short Width;

	// the image height.... 
	unsigned short Height;

	// the bits per pixel of the image, 8,16,24 or 32
	unsigned char BPP;

	// ignore this field.... 0
	unsigned char Rubbish2;
};
typedef struct Header Header;


// The image header
Header ImageHeader;

// the Pixel data
unsigned char* ImageData = NULL;

// a union to check the byte order of 2 byte variables
union Check
{	
	// 1 x 2 byte variable
	unsigned short s;

	// 2 x 1 byte variables
	unsigned char c[2];
};
typedef union Check Check;

// returns 1 if PC, 0 if irix or mac
int TestByteOrder() 
{
	Check check;

	check.s = 1;

	if( check.c[0] == 1 )
	{
		return 1;
	}
	return 0;
}

// swaps the bytes in a 2 byte variable
void SwapBytes(unsigned short* s)
{
	unsigned char* c = (unsigned char*)s;
	unsigned char temp = c[0];
	c[0] = c[1];
	c[1] = temp;
}


unsigned int MakeTexture();

unsigned int LoadTexture(char* filename)
{
	unsigned int PixelDataSize,i;

	FILE* fp = fopen(filename,"rb");


	if(fp == NULL)
	{		
		printf("Could not open the file %s\n",filename);
		return 0;
	}


	fread(&ImageHeader,1,sizeof(Header),fp);

	if( TestByteOrder() == 0)
	{
		SwapBytes(&ImageHeader.Height);
		SwapBytes(&ImageHeader.Width);
	}

	PixelDataSize = ImageHeader.Height * 
					ImageHeader.Width * 
					ImageHeader.BPP/8;


	ImageData = (unsigned char*)malloc(PixelDataSize);

	assert(ImageData);

	fread(ImageData,PixelDataSize,sizeof(unsigned char),fp);

	for(i=0;i < PixelDataSize; i+= ImageHeader.BPP/8 )
	{	
		unsigned char temp = ImageData[i];
		ImageData[i] = ImageData[i+2];
		ImageData[i+2] = temp;
	}

	fclose(fp);

	return MakeTexture();
}





unsigned int MakeTexture()
{
	unsigned int texobj;

	glGenTextures(1,&texobj);
	glBindTexture(GL_TEXTURE_2D,texobj);
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

	/* repeat the texture in both directions */
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	/* use bi-linear filtering to smooth the texture nicely. */
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	if(ImageHeader.BPP == 24)
	{
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,ImageHeader.Width,ImageHeader.Height,
			0,GL_RGB,GL_UNSIGNED_BYTE,ImageData);
	}
	else
	if(ImageHeader.BPP == 32)
	{
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,ImageHeader.Width,ImageHeader.Height,
			0,GL_RGBA,GL_UNSIGNED_BYTE,ImageData);
	}
	
	free( ImageData );
	ImageData = NULL;

	return texobj;
}

