/*----------------------------------------------------------------------------------*\
*	file:	TextureLoader.c
\*----------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------*\
*		Includes
\*----------------------------------------------------------------------------------*/
#include "TextureLoader.h"
#include <stdlib.h>
#include <GL/glut.h>
#include <stdio.h>

#pragma pack(1)
/*
 *	tga header structure is 18bytes, however, it also includes a lot of stuff 
 *  we dont need therefore we call it unused and unused2, and ignore it.
 */
struct {
	unsigned char unused[12];
	unsigned short w;
	unsigned short h;
	unsigned char  ImagePixelSize;
	unsigned char unused2;
} TGA_Header;
/*
 *	array used to store texture data
 */
unsigned char *PixelDataArray=NULL;

/*
 *	load texture as according to its filename
 */
unsigned int loadTexture (char* filename)
{
	GLenum Format ;
	unsigned int data_size,i;
	unsigned int tex_obj;
/*
 *	open and read binary file however if texture could not be loaded
 *	return
 */
	FILE* fp = fopen (filename, "rb");
	if (fp == NULL) {
		return;
	}
/*
 *	read the tga header
 */
	fread(&TGA_Header,1,18,fp);

	/*
	 *	bytes the image needs for it to be held in memory
	 *
	 *	Width * Height * (Bytes per pixel)
	 */
	data_size = TGA_Header.ImagePixelSize/8 * TGA_Header.w * TGA_Header.h;

	/*
	 *	Allocate enough space to hold pixel data
	 */
	PixelDataArray = (unsigned char *)malloc(data_size);

	/*
	 *	Read the data
	 */
	fread(PixelDataArray, data_size,1, fp);
/*
 *	close the file
 */
	fclose(fp);

/*
 *	convert from bgr to rgb values (24 bit or 32 bit both multiples of eight).
 *  I'm also taking using this for bgra to rgba
 */
	for(i=0;i<data_size;i+=(TGA_Header.ImagePixelSize/8))
	{
		unsigned char temp = PixelDataArray[i];
		PixelDataArray[i] = PixelDataArray[i+2];
		PixelDataArray[i+2] = temp;
	}

	glGenTextures(1,&tex_obj);
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

	/* set the current texture to the one just created */
	glBindTexture (GL_TEXTURE_2D, tex_obj);

	/* repeat the texture in both directions */
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	/* use bi-linear filtering to smooth the texture nicely. */
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

/*
 *	application can open either 24 bit or 32 bit tga image files
 */
	if (TGA_Header.ImagePixelSize == 24) {
		Format = GL_RGB;
	}
	else 
	if (TGA_Header.ImagePixelSize == 32) {
		Format = GL_RGBA;
	}
	else
	{
		/* format not supported */
		exit(0);
	}

	/* 
	*	load the texture data into the graphics hardware
	*/
	glTexImage2D(GL_TEXTURE_2D,0,Format,TGA_Header.w,TGA_Header.h,0,Format,GL_UNSIGNED_BYTE,PixelDataArray);

/*
 *	free memory
 */
	free(PixelDataArray);
/*
 *	return texture object
 */
	return tex_obj;
}