/*----------------------------------------------------------------------------------*\
*	file:	Main.c
\*----------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------*\
*		Includes
\*----------------------------------------------------------------------------------*/
#include "TextureLoader.h"
#include "ModelLoader.h"

#ifdef WIN32
	#include <windows.h>
	char ModelFile[260]={0}; 
	char TextureFile[260]={0}; 
#else
	char ModelFile[260]="horse.obj"; 
	char TextureFile[260]="fur1.tga"; 
#endif


#include <GL/glut.h>

/*----------------------------------------------------------------------------------*\
*		Globals
\*----------------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------------*/
/*
 *	define and create a single window structure
 */
struct {
	int w;
	int h;
} Window;

void resize (int w, int h)
{	
	/*
	 *	Store window width and height
	 */
	Window.w = w;
	Window.h = h;

	glViewport(0,0,Window.w,Window.h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45,w/h,1,100);
	glMatrixMode(GL_MODELVIEW);
}
/*----------------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------------*/
/*
 *	define and set default values for display options
 */
struct {
	int wire;
	int light;
	int texture;
} DisplayOptions = {
	0,1,1	
};

/*
 *	parameters for function include a single character KEY input from the keyboard
 *	two integer values for x and y
 */
void Keyboard(unsigned char key,int x,int y)
{
	/*
	 *	when four is pressed statement is true so wire mode enabled
	 */
	if (key == '4') {
		DisplayOptions.wire = 1;
	/*
	*	when five is pressed statement is false so wire mode disabled
	*/
	}
	if (key == '5') {
		DisplayOptions.wire = 0;
	}
	/*
	*	when six is pressed lights are enabled
	*/
	if (key == '6') {
		DisplayOptions.light = 1;
	}
	/*
	*	when seven is pressed lights are disabled
	*/
	if (key == '7') {
		DisplayOptions.light = 0;
	}
	/*
	*	when eight is pressed tetxure is enabled
	*/
	if (key == '8') {
		DisplayOptions.texture = 1;
	}
	/*
	*	when nine is pressed tetxure is disabled
	*/
	if (key == '9') {
		DisplayOptions.texture = 0;
	}
	glutPostRedisplay ();
}

/*
 *	function to set display options for application
 */
void SetDisplayOptions()
{
	/*
	*	Set wireframe mode
	*/
	if(DisplayOptions.wire==1)
		/*
		 *	when above statement is true then polygons are drawn in lines 
		 *	otherwise draw polygons filled in
		 */
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	else
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	/*
	*	Set lighting mode
	*/
	if (DisplayOptions.light==1) {
		/*
		 *	when the light statement above is true enable lights, otherwise
		 *	turn lights off!
		 */
		glEnable(GL_LIGHTING);
	}
	else	{
		glDisable(GL_LIGHTING);
	}
	/*
	*	Set texture mode
	*/
	if (DisplayOptions.texture ==1) {
		/*
		 *	when texture statement above is true then show texture, otherwise
		 *	dont show texture!
		 */
		glEnable(GL_TEXTURE_2D);
	}
	else	{
		glDisable(GL_TEXTURE_2D);
	}
}
/*----------------------------------------------------------------------------------*/

/*
*	define and set default values for camera movements
*/
struct {
	float zoom;
	float rotx;
	float roty;
	float trackx;
	float tracky;
} CameraMovement = 
{
	0,0,0,0,0	
};
/*
 *	set up perspective camera and respective movements
 */
void SetCameraPosition()
{
	glLoadIdentity();
	/*
	*	set camera pos
	*	looks down z axis
	*	positioned at the origin
	*	up vector is in the y axis
	*/
	gluLookAt(0,0,3,
		0,0,0,
		0,1,0);
	/*
	*	zoom along z axis
	*/
	glTranslatef(0,0,CameraMovement.zoom);
	/*
	 *	rotate camera around x axis
	 */
	glRotatef(CameraMovement.rotx,1,0,0);
	/*
	*	rotate camera around y axis
	*/
	glRotatef(CameraMovement.roty,0,1,0);
	/*
	*	translate camera around x and y axis
	*/
	glTranslatef(CameraMovement.trackx,CameraMovement.tracky,0);
}


/*----------------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------------*/
/*
*	define and set default values for mouse buttons and alt button 
*	on keyboard
*/
struct  {
	int lastx;
	int lasty;
	int left;
	int right;
	int middle;
	int altPressed ;
}Mouse = {
	0	
};
/*
*	mouse button function 
*	parameter include which button is pressed
*	what state that button is in
*	the mouse x, y co-ordinates
*/
void MouseButton(int button,int state,int x,int y)
{
	/*
	 *	last x and y values get updated to current values of x and y
	 */
	Mouse.lastx = x;
	Mouse.lasty = y;
/*
 *	alt button is pressed 
 */
	if(glutGetModifiers() & GLUT_ACTIVE_ALT)
	{
		Mouse.altPressed=1;
	}
	else
		Mouse.altPressed=0;
/*
 *	if mouse buttons pressed
 */
	if(state == GLUT_DOWN)
	{
		switch(button) {
		case GLUT_LEFT_BUTTON:
			/*
			 *	if middle mouse button pressed change cursor for zoom
			 */
			if(Mouse.middle==1)
			{
				/*
				 *	change cursor for zoom
				 */
				glutSetCursor(GLUT_CURSOR_LEFT_RIGHT);
			}
			else
			{
				/*
				 *	otherwise change cursor for rotate
				 */
				glutSetCursor(GLUT_CURSOR_CYCLE);
			}
			/*
			 *	because only the left mouse button would be pressed
			 *	that is used for rotating around the object
			 */
			Mouse.left=1;
			break;
		case GLUT_RIGHT_BUTTON:
			Mouse.right=1;
			break;
		default:
			/*
			 *	if left mouse button is down then cursor changes for zoom
			 */
			if(Mouse.left==1)
			{
				/*
				 *	zoom cursor
				 */
				glutSetCursor(GLUT_CURSOR_LEFT_RIGHT);
			}
			else{
				/*
				 *	otherwise if it is not pressed and only middle mouse 
				 *	button is pressed then cursor changes for translate  
				 */
				glutSetCursor(GLUT_CURSOR_SPRAY);
			}
			/*
			 *	middle mouse button and alt changes cursor 
			 *	and used to translate around object
			 */
			Mouse.middle=1;
			break;
		}
	}
	else
	{
		switch(button) {
	
		/* left button released */
		case GLUT_LEFT_BUTTON:
			Mouse.left=0;
			break;
		/* right button released */
		case GLUT_RIGHT_BUTTON:
			Mouse.right=0;
			break;
		/* middle button released */
		default:
			Mouse.middle=0;
			break;
		}

		glutSetCursor(GLUT_CURSOR_INHERIT);
	}
	glutPostRedisplay ();
}

/*
 *	function for mouse motion
 *	parameters are for the x and y co-ordinates of the cursor
 */
void Motion(int x,int y)
{
	/*
	 *	change in x and y values will equal the last x and 
	 *	y values of the mouse and will be updated to the new values
	 */
	int dx = x-Mouse.lastx;
	int dy = y-Mouse.lasty;
	Mouse.lastx = x;
	Mouse.lasty = y;

	/*
	*	if the alt button is pressed
	*/
	if (Mouse.altPressed) {
		/*
		*	and the left mouse button and the middle mouse button 
		*	are also pressed then zoom 
		*/
		if(Mouse.left==1 && Mouse.middle==1)
		{
			CameraMovement.zoom+=(0.02f*dx);
		}
		else
			/*
			*	or if alt is pressed and the middle mouse button 
			*	are pressed then translate along the x and y axis
			*/
		if(Mouse.middle==1)
		{
			CameraMovement.trackx +=(0.01f*dx);
			CameraMovement.tracky -=(0.01f*dy);
		}
		else
			/*
			*	or if alt is pressed and the left mouse button 
			*	are pressed then rotate around the x and y axis
			*/
		if(Mouse.left==1)
		{
			CameraMovement.rotx+=(dy);
			CameraMovement.roty +=(dx);
		}
	}
	glutPostRedisplay ();
}

/*----------------------------------------------------------------------------------*/

/*
 *	id number for the model
 */
int model;


void init()
{
	/*
	*	enable depth testing (z-buffer)
	*/
	glEnable(GL_DEPTH_TEST);
	/*
	 *	load the model file
	 */
	model=loadmod(ModelFile);
	/*
	 *	enable a light
	 */
	glEnable(GL_LIGHT0);
	/*
	 *	load the texture file
	 */
	loadTexture(TextureFile);
}


/*
 *	structure to draw screen
 */
void draw ()
{
	/*
	*	Clear the screen
	*/
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
/*
 *	set our display options 
 */
	SetDisplayOptions();
	/*
	*	set camera
	*/
	SetCameraPosition();
	/*
	*	draw model
	*/
	drawmod(model);
	/*
	*	swap buffers
	*/
	glutSwapBuffers();
}

#ifdef WIN32
void GetFile(char* buffer)
{
	OPENFILENAME ofn = {0};
	ofn.lpstrTitle  = "Open file";
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFile = buffer;
	ofn.nMaxFile = 260;
	GetOpenFileName(&ofn);
}
#endif

/*
 *	create and initialise window
 */
int main(int argc, char** argv)
{
#ifdef WIN32
	GetFile(ModelFile);
	GetFile(TextureFile);
#endif
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE|GLUT_DEPTH);
	glutInitWindowSize(500,500);
	glutCreateWindow("sam");


	init();

/*
 *	set glut callback functions
 */
	glutDisplayFunc(draw);
	glutReshapeFunc(resize);
	glutMouseFunc(MouseButton);
	glutMotionFunc(Motion);
	glutKeyboardFunc(Keyboard);

	glutMainLoop();
	return 0;
	
}