///-----------------------------------------------------------------------------
/// 
/// \file	
/// \author	
/// \brief	
/// 
///-----------------------------------------------------------------------------

#include "model.h"
#include <assert.h>
#include <fstream>

XModel::XModel(void) : m_TextureObject(0)
{
}

XModel::~XModel(void)
{
	// delete tesxtures
	glDeleteTextures(m_NumTextures,&m_TextureObject);

	// delete all lights
	{
		XItLight it = m_Lights.begin();
		for( ; it != m_Lights.end(); ++it )
		{
			delete *it;
		}
	}

	// delete all meshes
	{
		XItMesh it = m_Meshes.begin();
		for( ; it != m_Meshes.end(); ++it )
		{
			delete *it;
		}
	}
}

/// this Function enables all of the lights in the model
void XModel::EnableLights()
{
	// return false if no lights exist
	if (m_Lights.empty()) {
		return;
	}

	// enable lighting
	glEnable(GL_LIGHTING);

	// enable all lights
	XItLight it = m_Lights.begin();
	for(;it!=m_Lights.end();++it)
	{
		(*it)->Enable();
	}
}

/// this function disables all of the lights in the model
void XModel::DisableLights()
{
	// return if no lights exist
	if (m_Lights.empty()) {
		return;
	}
	
	// disable lighting
	glDisable(GL_LIGHTING);

	// disable each light
	XItLight it = m_Lights.begin();
	for(;it!=m_Lights.end();++it)
	{
		(*it)->Disable();
	}
}


/// a function to draw the geometry data in the model
void XModel::gl()
{
	glPushMatrix();

		EnableLights();

		// draw each mesh
		XItMesh it = m_Meshes.begin();
		for(;it!=m_Meshes.end();++it)
		{
			(*it)->gl();
		}

		DisableLights();

	glPopMatrix();
}


/// \brief	a function to load a file into the model
/// \param	filename	-	the name of the file to load
/// \return	true if OK
/// 
bool XModel::Load(const char* const filename)
{
	char buffer[32];
	std::ifstream ifs(filename);

	if (ifs) {

		while (!ifs.eof()) {
			ifs >> buffer;
			if (ifs.eof()) {
				break;
			}
			std::cout << buffer << std::endl;

			if (buffer[0] == 'M' &&
				buffer[1] == 'E' &&
				buffer[2] == 'S' &&
				buffer[3] == 'H') 
			{
				XMesh* pm = new XMesh;
				assert (pm);
				ifs >> *pm;
				m_Meshes.push_back(pm);
			}
			else
			if (buffer[0] == 'L' &&
				buffer[1] == 'I' &&
				buffer[2] == 'G' &&
				buffer[3] == 'H' &&
				buffer[4] == 'T' ) 
			{
				XLight* pl = new XLight( static_cast<unsigned int>(m_Lights.size()));
				assert (pl);
				ifs >> *pl;
				m_Lights.push_back(pl);
			}
			else
			{
#ifdef DEBUG
				std::cerr << "[ERROR] FileIO Error while trying to read " 
						  << filename << std::endl;
#endif
				ifs.close();
				return false;
			}
		}
		ifs.close();
	}
	return true;
}
