///-----------------------------------------------------------------------------
/// 
/// \file	Model.h
/// \author	Rob Bateman
/// \brief	This file defines the XModel class. basicaly this class will load the 
///			contents of a mod file. This version merely holds lights and 
///			meshes in one place and is responsible for drawing them.
/// 
///-----------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////
/// The XModel class basically holds the contents of a read file. This may
/// contain a number of 


#pragma once

//////////////////////////////////////////////////////////////////////////

#include <vector>
#include <list>
#include "./Light.h"
#include "./Mesh.h"
#include "./TextureFile.h"

//////////////////////////////////////////////////////////////////////////


/// a list of lights type
typedef std::vector< XLight* > XLightList;

/// an iterator for a light list
typedef XLightList::iterator XItLight;



/// a list of lights type
typedef std::vector< XMesh* > XMeshList;

/// an iterator for a light list
typedef XMeshList::iterator XItMesh;


//////////////////////////////////////////////////////////////////////////

class XModel
{
public:

	/// ctor
	XModel(void);

	/// dtor
	~XModel(void);

	/// a function to draw the geometry data in the model
	void gl();

	/// a function to load a file into the model
	bool Load(const char* const filename);

	/// is the model textured?
	bool IsTextured() const {
		return m_TextureObject != 0 ;
	}

	/// does the model contain lights?
	bool IsLit() const {
		return m_Lights.size() != 0;
	}

protected:

	/// this Function enables all of the lights in the model
	void EnableLights();

	/// this function disables all of the lights in the model
	void DisableLights();

	/// a texture object for all of the textures in the model
	unsigned int m_TextureObject;

	/// the number of textures contained (only used to delete the texture objects later)
	unsigned int m_NumTextures;

	/// a list of lights
	XLightList m_Lights;

	/// a list of meshes
	XMeshList m_Meshes;


};
