///-----------------------------------------------------------------------------
/// 
/// \file	TextureFile.h
/// \author	Rob Bateman
/// \brief	the XTextureFile class holds a list of all loaded textures from the 
///			specified file.
/// 
///-----------------------------------------------------------------------------

#pragma once

#include <vector>
#include "Texture.h"

class XTextureFile : protected std::vector< XTexture* >
{
public:

	/// ctor
	XTextureFile(void);

	/// dtor
	~XTextureFile(void);

	/// returns a pointer to the texture data
	XTexture* GetTexture(unsigned int id) const 
	{
		if(id == 0)
			return NULL;

		if (id-1 < size()) {
			return (*this)[id-1];
		}
		return NULL;
	}

	/// this function converts all image data into a single openGL texture object. 
	/// to use each texture, use tex_obj + n to get the n'thy texture object
	/// 
	bool TurnIntoTextures(unsigned int& tex_obj);

	/// returns the number of textures in the file
	unsigned int GetNumTextures() const {
		return static_cast<unsigned int>(size());
	}

	/// this function loads the specified file
	bool Load(const char* const filename);


};
