

#define NT_APP


// include each header for each class you need. Gets a bit laborious later on...
//
#include <maya/MImage.h>
#include <maya/MLibrary.h>
#include <maya/MGlobal.h>

#include <maya/MFileIO.h>
#include <maya/MFloatMatrix.h>
#include <maya/MFloatVector.h>
#include <maya/MFloatVectorArray.h>
#include <maya/MMatrix.h>
#include <maya/MPlug.h>
#include <maya/MPoint.h>
#include <maya/MPointArray.h>

#include <maya/MFnAmbientLight.h>
#include <maya/MFnAreaLight.h>
#include <maya/MFnDagNode.h>
#include <maya/MFnDirectionalLight.h>
#include <maya/MFnLight.h>
#include <maya/MFnMesh.h>
#include <maya/MFnPointLight.h>
#include <maya/MFnSpotLight.h>
#include <maya/MFnTransform.h>

#include <maya/MItDependencyNodes.h>

#include "./Light.h"
#include "./Mesh.h"
#include "./Texture.h"

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <map>

// link to some fairly funky maya libs. NOTE: The Image library only became part of maya
// from version 4.01 onwards. It is not available in version 4.0
//
#pragma comment(lib,"Foundation.lib")
#pragma comment(lib,"OpenMaya.lib")
#pragma comment(lib,"Image.lib")



int main(int argc,char** argv)
{


	// if the argument count is one, then no files were specified
	//
	if (argc==1) {
		std::cerr << "[ USAGE ] Exporter <infile.mb> { <outfile.ext> }" << std::endl;
		return 0;
	}

	std::string outFilePrefix;


	// var to hold the output file name
	std::string outfilename;

	// if only one argument was specified, I'll assume that it is a maya binary
	// or ascii file, in which case use this - 'mb' or 'ma' and add a 'mod' extension
	// to the end of the filename 
	// 
	outFilePrefix = argv[1];
	outFilePrefix.resize(outFilePrefix.size()-2);
	
	if (argc == 2 )
	{

		outfilename = outFilePrefix;
		outfilename += "mod";
	}
	else
	{
		// if outfile file was specified, use that
		//
		outfilename = argv[2];
	}

	outFilePrefix += "tex";


	// initialise the maya library - This basically starts up Maya
	// 
	MLibrary::initialize(argv[0]);




	// open the file that the user specified 
	//
	if( MS::kSuccess == MFileIO::open(argv[1],NULL,true) )
	{

		// do the texture file for a laugh....
		//
		DoTextures(outFilePrefix);

		// create an output file stream
		//
		std::ofstream ofs(outfilename.c_str());

		// make sure it opened OK
		//
		if (!ofs) {
			std::cerr << "Could not open output file " << outfilename << std::endl;
			MLibrary::cleanup();
			return 1;
		}


		{
			// iterate through the mesh nodes in the scene
			//
			MItDependencyNodes itDep;

			// we have to keep iterating until we get through all of the nodes in the scene
			//
			while (!itDep.isDone()) 
			{
				// 
				MObject item = itDep.item();

				switch(item.apiType()) 
				{
				case MFn::kMesh:
					OutputMesh(ofs,item);
					break;

				case MFn::kLight:
				case MFn::kAreaLight:
				case MFn::kAmbientLight:
				case MFn::kDirectionalLight:
				case MFn::kPointLight:
				case MFn::kSpotLight:
					OutputLight(ofs,item);
					break;

				default:
					break;
				}


				// move to next node
				itDep.next();
			}
		}


		// close the output file
		ofs.close();
	}
	else
	{
		std::cerr << "Could not open file " << argv[1] << std::endl;
	}


	// cleanup maya data

	MLibrary::cleanup();

	return 0;
}