#include <stdlib.h>
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif
#include <GL/GL.h>
#include <GL/glut.h>
#include "bone.h"

XBone::XBone(void)
{
}

XBone::~XBone(void)
{
}
void XQuaternion::AsMatrix(OUT float* matrix)
{
	// sacrafice a few bytes to pre-calculate some values 
	float xx = x*x;
	float xy = x*y;
	float xz = x*z;
	float xw = x*w;
	float yy = y*y;
	float yz = y*z;
	float yw = y*w;
	float zz = z*z;
	float zw = z*w;

	// [0] -> [3]
	*(  matrix) = 1.0f - 2.0f * (yy+zz);
	*(++matrix) =        2.0f * (xy-zw);
	*(++matrix) =        2.0f * (xz+yw);
	*(++matrix) = 0.0f;

	// [4] -> [7]
	*(++matrix) =        2.0f * (xy+zw);
	*(++matrix) = 1.0f - 2.0f * (xx+zz);
	*(++matrix) =        2.0f * (yz-xw);
	*(++matrix) = 0.0f;
	
	// [8] -> [11]
	*(++matrix) =        2.0f * (xz-yw);
	*(++matrix) =        2.0f * (yz+xw);
	*(++matrix) = 1.0f - 2.0f * (xx+yy);
	*(++matrix) = 0.0f;
	
	// [12] -> [15]
	*(++matrix) = 0.0f;
	*(++matrix) = 0.0f;
	*(++matrix) = 0.0f;
	*(++matrix) = 1.0f;
}



const float episilon = 0.001f;

/// shouldn't use the == operator for floats...
/// 
#define FCOMPARE(X_,Y_) (((X_)>((Y_) - episilon)) && ( (X_) < ((Y_)-episilon)))

/// a function to spherically interpolate the two quaternions 
//
XQuaternion SLERP(const XQuaternion &q1,const XQuaternion &q2,const float &t)
{
	XQuaternion Second;
	XQuaternion Final;

	// if two quarternions are the same
	if ( FCOMPARE( q1.x, q2.x ) &&
		 FCOMPARE( q1.y, q2.y ) &&
		 FCOMPARE( q1.z, q2.z ) &&
		 FCOMPARE( q1.w, q2.w ) ) 
	{
		return q1;
	}

	// perform dot product
	float product = ( q1.x * q2.x ) + 
					( q1.y * q2.y ) +
					( q1.z * q2.z ) +
					( q1.w * q2.w ) ;

	// if the angle is greater than 90degress, negate quaternion
	if ( product < 0.001f ) 
	{
		Second = XQuaternion(-q2.x,-q2.y,-q2.z,-q2.w);
		product = -product;
	}
	else
	{
		Second = q2;
	}

	// 
	float interpolating_t = t;
	float interpolating_invt = 1.0f-t;

		float theta = static_cast<float>( acos(product) );
		float sinTheta = static_cast<float>( sin(theta) );

	if(sinTheta != 0.0f) 
	{
		interpolating_invt = static_cast<float>( sin(interpolating_invt * theta) / sinTheta   );
		interpolating_t = static_cast<float>( sin(interpolating_t * theta) / sinTheta   );
	}

	Final.x = (interpolating_invt * q1.x) + (interpolating_t * Second.x);
	Final.y = (interpolating_invt * q1.y) + (interpolating_t * Second.y);
	Final.z = (interpolating_invt * q1.z) + (interpolating_t * Second.z);
	Final.w = (interpolating_invt * q1.w) + (interpolating_t * Second.w);

	return Final;
}


std::istream& operator >> (std::istream& ifs,XBone &b)
{
	char buffer[64];
	int temp;
	ifs >> buffer; // joint name
	b.name = buffer;
	std::cout << buffer << std::endl;
	ifs	>> buffer // "id"
		>> b.m_ID 
		>> buffer // "parent"
		>> b.m_ParentID
		>> buffer // "children"
		>> temp;

	b.m_ChildJoints.resize(temp);

	{
		XBone::ChildIt it = b.m_ChildJoints.begin();

		for( ;it != b.m_ChildJoints.end(); ++it )
		{
			ifs >> *it;
		}
	}

	ifs >> buffer // "ParentedNodes"
		>> temp;

	b.m_ChildObjects.resize(temp);

	{
		XBone::ChildIt it = b.m_ChildObjects.begin();

		for( ;it != b.m_ChildObjects.end(); ++it )
		{
			ifs >> *it;
		}
	}

	ifs >> buffer // "ParentedNodes"
		>> b.m_Scale.x >> b.m_Scale.y >> b.m_Scale.z
		>> buffer // "translation"
		>> b.m_Translation.x >> b.m_Translation.y >> b.m_Translation.z
		>> buffer // "RotationOrient"
		>> b.m_RotationOrient
		>> buffer // "Rotation"
		>> b.m_Rotation
		>> buffer // "JointOrient"
		>> b.m_JointOrient;



	
	return ifs;
}

void XBone::DrawBones( 
					   std::vector< XBone*   > &Bones , 
					   std::vector< XObject* > &Objects,
					   std::vector< XMaterial*> &Materials,XQuaternion q1  ) 
{
		float matrix[16];

	glPushMatrix();

		XQuaternion q2 = m_JointOrient;

		// perform dot product
		float product = ( q1.x * q2.x ) + 
			( q1.y * q2.y ) +
			( q1.z * q2.z ) ;
		if ( product < 0.001f ) {
			q2 = XQuaternion(-q2.x,-q2.y,-q2.z,q2.w);
		}


		if(0!=m_ParentID) {
			glDisable(GL_LIGHTING);
				glBegin(GL_LINES);
					glVertex3f(0.001f,0.001f,0.001f);		
					glVertex3f(m_Translation.x,m_Translation.y,m_Translation.z);
				glEnd();
			glEnable(GL_LIGHTING);
		}

		q2.AsMatrix(matrix);

		matrix[12] = m_Translation.x;
		matrix[13] = m_Translation.y;
		matrix[14] = m_Translation.z;

		// apply rotation
		glMultMatrixf(matrix);


		(-m_Rotation).AsMatrix(matrix);

		// apply rotation
		glMultMatrixf(matrix);

		// scale is applied first
		glScalef(m_Scale.x,m_Scale.y,m_Scale.z);


		for( ChildIt it = m_ChildObjects.begin(); it != m_ChildObjects.end(); ++it )
		{
			std::vector< XObject* >::iterator itO = Objects.begin();
			for( ; itO != Objects.end(); ++itO )
			{
				if( (*itO)->GetID() == *it )
				{
					(*itO)->gl(Materials);
				}
			}
		}

		for( ChildIt it = m_ChildJoints.begin(); it != m_ChildJoints.end(); ++it )
		{
			std::vector< XBone* >::iterator itB = Bones.begin();
			for( ; itB != Bones.end(); ++itB )
			{
				if( (*itB)->m_ID == *it )
				{
					(*itB)->DrawBones(Bones,Objects,Materials,q2);
				}
			}
		}


	glPopMatrix();
}


