///-----------------------------------------------------------------------------
/// 
/// \file	
/// \author	
/// \brief	
/// 
///-----------------------------------------------------------------------------
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif
#include <GL/GL.h>
#include "Model.h"
#include "Mesh.h"
#include "TextureFile.h"
#include <assert.h>
#include <fstream>

XModel::XModel(void) : m_TextureObject(0),m_NumTextures(0)
{
}

XModel::~XModel(void)
{
	// delete tesxtures
	glDeleteTextures(m_NumTextures,&m_TextureObject);


	// delete all meshes
	{
		XIt it = m_Objects.begin();
		for( ; it != m_Objects.end(); ++it )
		{
			delete *it;
		}
	}
}

/// this Function enables all of the lights in the model
void XModel::EnableLights()
{
	// return false if no lights exist
	if (m_Lights.empty()) {
		return;
	}

	// enable lighting
	glEnable(GL_LIGHTING);

	// enable all lights
	XItLight it = m_Lights.begin();
	for(;it!=m_Lights.end();++it)
	{
		(*it)->Enable();
	}
}

/// this function disables all of the lights in the model
void XModel::DisableLights()
{
	// return if no lights exist
	if (m_Lights.empty()) {
		return;
	}
	
	// disable lighting
	glDisable(GL_LIGHTING);

	// disable each light
	XItLight it = m_Lights.begin();
	for(;it!=m_Lights.end();++it)
	{
		(*it)->Disable();
	}
}


/// a function to draw the geometry data in the model
void XModel::gl()
{
	glPushMatrix();

		EnableLights();

		// draw each mesh
		XItBone it = m_Bones.begin();
		for(;it!=m_Bones.end();++it)
		{
			// if a root node
			if((*it)->GetParentID() == 0)
				(*it)->DrawBones(m_Bones,m_Objects,m_Materials,(m_Bones[0])->m_Rotation);
		}

		DisableLights();

	glPopMatrix();
}


void XModel::Update(float dt)
{
	XItAnim it = m_Anims.begin();
	for( ; it != m_Anims.end() ; ++it)
	{

		XItBone it2 = m_Bones.begin();

		for( ;it2 != m_Bones.end(); ++it2 )
		{
			if ( *it && *it2 && (*it2)->GetID() == (*it)->GetID() ) {
				(*it)->Update( *(*it2), dt );
			}
		}
	
	}
}

/// \brief	a function to load a file into the model
/// \param	filename	-	the name of the file to load
/// \return	true if OK
/// 
bool XModel::Load(const char* const filename)
{
	{
		// first of all, see if there is a texture file
		std::string texfile = filename;
		texfile.resize(texfile.size()-3);
		texfile += "tex";

		// load the texture file
		XTextureFile texLoader;
		if(texLoader.Load(texfile.c_str()))
		{
			// if OK, pass all textures to openGL
			if(texLoader.TurnIntoTextures(m_TextureObject))
			{
				// store number of textures
				m_NumTextures = texLoader.GetNumTextures();
			}
		}
	}

	char buffer[32];
	std::ifstream ifs(filename);

	if (ifs) {

		while (!ifs.eof()) {
			ifs >> buffer;
			if (ifs.eof()) {
				break;
			}
			std::cout << buffer << std::endl;

			if (buffer[0] == 'B' &&
				buffer[1] == 'O' &&
				buffer[2] == 'N' &&
				buffer[3] == 'E') 
			{
				XBone* pb = new XBone;
				assert (pb);
				ifs >> *pb;
				m_Bones.push_back(pb);
			}
			else

			if (buffer[0] == 'B' &&
				buffer[1] == 'o' &&
				buffer[2] == 'n' &&
				buffer[3] == 'e') 
			{
				XBoneAnimation* pb = new XBoneAnimation;
				assert (pb);
				ifs >> *pb;
				m_Anims.push_back(pb);
			}
			else
			if (buffer[0] == 'L' )
			{
				if(	buffer[1] == 'I' &&
					buffer[2] == 'G' &&
					buffer[3] == 'H' &&
					buffer[4] == 'T' ) 
				{
					XLight* pl = new XLight( static_cast<unsigned int>(m_Lights.size()));
					assert (pl);
					ifs >> *pl;
					m_Lights.push_back(pl);
					m_Objects.push_back(pl);
				}
				else
				if(	buffer[1] == 'A' &&
					buffer[2] == 'M' &&
					buffer[3] == 'B' &&
					buffer[4] == 'E' &&
					buffer[5] == 'R' &&
					buffer[6] == 'T') 
				{
					XLambert* pp = new XLambert;
					assert (pp);
					ifs >> *pp;
					if (m_TextureObject!=0) {

						pp->SetTextureStartNum( m_TextureObject );
					}
					m_Materials.push_back(pp);
				}
			}
			else
			if (buffer[0] == 'M' &&
				buffer[1] == 'E' &&
				buffer[2] == 'S' &&
				buffer[3] == 'H') 
			{
				XMesh* pm = new XMesh;
				assert (pm);
				ifs >> *pm;
				m_Objects.push_back(pm);
			}
			else
			if (buffer[0] == 'P' &&
				buffer[1] == 'H' &&
				buffer[2] == 'O' &&
				buffer[3] == 'N' &&
				buffer[4] == 'G' ) 
			{
				XPhong* pp = new XPhong;
				assert (pp);
				ifs >> *pp;
				if (m_TextureObject!=0) {

					pp->SetTextureStartNum( m_TextureObject );
				}
				m_Materials.push_back(pp);
			}
			else
			{
#ifdef DEBUG
				std::cerr << "[ERROR] FileIO Error while trying to read " 
						  << filename << std::endl;
#endif
				ifs.close();
				return false;
			}
		}
		ifs.close();
	}
	return true;
}
