///-----------------------------------------------------------------------------
/// 
/// \file	Model.h
/// \author	Rob Bateman
/// \brief	This file defines the XModel class. basicaly this class will load the 
///			contents of a mod file. This version merely holds lights and 
///			meshes in one place and is responsible for drawing them.
/// 
///-----------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////
/// The XModel class basically holds the contents of a read file. This may
/// contain a number of 


#pragma once

//////////////////////////////////////////////////////////////////////////

#include <vector>
#include <list>
#include "./Light.h"
#include "./Material.h"
#include "./Object.h"
#include "./Bone.h"
#include "./BoneAnimation.h"
#include "./TextureFile.h"

//////////////////////////////////////////////////////////////////////////



/// a list of lights type
typedef std::vector< XBone* > XBoneList;

/// an iterator for a light list
typedef XBoneList::iterator XItBone;


/// a list of lights type
typedef std::vector< XLight* > XLightList;

/// an iterator for a light list
typedef XLightList::iterator XItLight;



/// a list of lights type
typedef std::vector< XObject* > XObjectList;

/// an iterator for a light list
typedef XObjectList::iterator XIt;




/// a list of lights type
typedef std::vector< XBoneAnimation* > XAnimList;

/// an iterator for a light list
typedef XAnimList::iterator XItAnim;


//////////////////////////////////////////////////////////////////////////

class XModel
{
public:

	/// ctor
	XModel(void);

	/// dtor
	~XModel(void);

	/// a function to draw the geometry data in the model
	void gl();

	/// a function to load a file into the model
	bool Load(const char* const filename);

	/// is the model textured?
	bool IsTextured() const {
		return m_TextureObject != 0 ;
	}

	/// does the model contain lights?
	bool IsLit() const {
		return m_Lights.size() != 0;
	}

	void Update(float dt);

protected:

	/// this Function enables all of the lights in the model
	void EnableLights();

	/// this function disables all of the lights in the model
	void DisableLights();

	/// a texture object for all of the textures in the model
	unsigned int m_TextureObject;

	/// the number of textures in the model
	unsigned int m_NumTextures;

	/// a list of materials
	XMaterialList m_Materials;

	/// a list of lights
	XLightList m_Lights;

	/// a list of meshes
	XObjectList m_Objects;

	/// an array of the models joints
	XBoneList m_Bones;

	/// an array of animation
	XAnimList m_Anims;
};
