//	
//	OK, now we are going to look at textures (note - not materials). Maya offers us
//	the really handy class MImage which allows us to load (though not save) any file format
//	supported by maya. This basically allows us to traverse the scene looking for file
//	texture nodes, read the filename, load the file and then output the data in our own
//	format. This may seem a bit long winded, however it basically gives us a HELL of a lot 
//	of freedom because we don't need to know how to load hectic file formats such as jpg
//	tiff etc. It also gives us an opportunity to check the texture size. 
//
//	OpenGL only supports textures that have dimension that is of power 2, ie 2,4,8,16,32 etc.
//	Now, if the texture file is not power of 2, we can simply resize it to a valid size and
//	print a warning. The constant g_DefaultTextureRes exists for this purpose. Currently
//	this is set to 128.
//
//	When it comes to storing this texture data we really don't want to store it in the same
//	file as the model data, mainly because there is FUCK loads of texture data, so it's better
//	to package all of the textures together into a single file. Basically to ensure that we
//	never loose textures or have problems with Full paths names when transferred to another 
//	computer, we will simply use an ID referencing system that will refer to a zero based 
//	index which represents the specific texture file.
//
//	err, bit pissed right now. Anyhow, For each image we want to store :
//
//	width	
//	height	
//	bpp		1/3/4
//
//	It's index is given by it's place in the file (starting at zero).
//
//	Now we could hold that data as 3 shorts, or 2 shorts and a bool. Thats a bit cack. 
//	I kinda like nice small fields, and a bit of data compaction. Instead of storing the
//	width in pixels, why not just store the power? With 4bits of data (a nibble) we can 
//	store a value between 0 and 15, ie 2^0 to 2^15, or 1 to 32768. So in 1byte we can store
//	the width and the height! If we then add 1byte to store the bpp we should have minimal
//	data use and a couple of bits left over (possibly compressed data flags?)	
//	

#define NT_APP


// include each header for each class you need. Gets a bit laborious later on...
//
#include <maya/MImage.h>
#include <maya/MLibrary.h>
#include <maya/MGlobal.h>

#include <maya/MDagPath.h>
#include <maya/MEulerRotation.h>
#include <maya/MFileIO.h>
#include <maya/MFloatMatrix.h>
#include <maya/MFloatVector.h>
#include <maya/MFloatVectorArray.h>
#include <maya/MMatrix.h>
#include <maya/MPlug.h>
#include <maya/MPoint.h>
#include <maya/MPointArray.h>
#include <maya/MQuaternion.h>
#include <maya/MAnimControl.h>

#include <maya/MFnAmbientLight.h>
#include <maya/MFnAreaLight.h>
#include <maya/MFnBlinnShader.h>
#include <maya/MFnDagNode.h>
#include <maya/MFnDirectionalLight.h>
#include <maya/MFnIkJoint.h>
#include <maya/MFnLambertShader.h>
#include <maya/MFnLight.h>
#include <maya/MFnMesh.h>
#include <maya/MFnPhongShader.h>
#include <maya/MFnPointLight.h>
#include <maya/MFnSpotLight.h>
#include <maya/MFnTransform.h>

#include <maya/MItDependencyNodes.h>
#include <maya/MItMeshPolygon.h>
#include <maya/MItMeshVertex.h>

#include <algorithm>
#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <map>

// link to some fairly funky maya libs. NOTE: The Image library only became part of maya
// from version 4.01 onwards. It is not available in version 4.0
//
#pragma comment(lib,"Foundation.lib")
#pragma comment(lib,"OpenMaya.lib")
#pragma comment(lib,"Image.lib")
#pragma comment(lib,"OpenMayaAnim.lib")



unsigned int ObjectRefID = 0;

typedef std::map<std::string, unsigned int> ObjectMap;
typedef ObjectMap::iterator ObjectIT;

/// a map of all useful transforms
ObjectMap g_Objects;



unsigned int GetObjectID(std::string name)
{
	// find the bone
	ObjectIT it = g_Objects.find(name);

	// if not found
	if (it == g_Objects.end()) 
	{
		// insert entry into bone map
		std::pair<ObjectIT,bool> pair_ = g_Objects.insert( std::make_pair( name , ++ObjectRefID  )  );
		
		return pair_.first->second;
	}
	return it->second;
}









unsigned int BoneRefID = 0;

struct CBoneRef 
{
	CBoneRef(std::string n) : name(n),m_Parent(0),id(++BoneRefID) {}
	CBoneRef(std::string n,MDagPath &p) : name(n),path(p),m_Parent(0),id(++BoneRefID) {}

	/// the name of the bone
	std::string name;

	/// the dag path to the node
	MDagPath    path;

	/// an array containing the names of lights, locator nodes, meshes etc
	std::vector<std::string> m_ParentedNodes;

	/// an array of child bones id's
	std::vector<unsigned int> m_Children;

	/// an array of child geometry id's
	std::vector<unsigned int> m_ChildObjects;

	/// the id of the parent node
	unsigned int m_Parent;

	/// the unique id of the bone
	unsigned int id;
};

typedef std::map<std::string, CBoneRef> BoneMap;
typedef BoneMap::iterator BoneIT;

/// an array of important transform nodes
std::vector<unsigned int> g_ImportantTransforms;

/// a map of all useful transforms
BoneMap g_Bones;


void AddChildToBone(unsigned int _child,unsigned int bone_id)
{
	// dont want to parent to myself
	if (_child == bone_id) {
		return;
	}

	BoneIT it = g_Bones.begin();

	for(; it != g_Bones.end(); ++it) {
		if (bone_id == it->second.id) {
			break;
		}
	}

	if (it != g_Bones.end()) {
		CBoneRef& joint = it->second;

		std::sort( joint.m_Children.begin(), joint.m_Children.end() );

		std::vector<unsigned int>::iterator itChildren = joint.m_Children.begin();

		for( ; itChildren != joint.m_Children.end() ; ++ itChildren ) {
			if ( *itChildren == _child) 
			{
				return;				
			}
		}
		joint.m_Children.push_back(_child);
	}
}


void AddObjectToBone(unsigned int _child,unsigned int bone_id)
{
	BoneIT it = g_Bones.begin();

	for(; it != g_Bones.end(); ++it) {
		if (bone_id == it->second.id) {
			break;
		}
	}

	if (it != g_Bones.end()) {
		CBoneRef& joint = it->second;

		std::sort( joint.m_ChildObjects.begin(), joint.m_ChildObjects.end() );

		std::vector<unsigned int>::iterator itChildren = joint.m_ChildObjects.begin();

		for( ; itChildren != joint.m_ChildObjects.end() ; ++ itChildren ) {
			if ( *itChildren == _child) 
			{
				return;				
			}
		}
		joint.m_ChildObjects.push_back(_child);
	}
}




unsigned int BuildJointRef(MObject &obj,bool doChildren = true)
{
	if (obj.hasFn(MFn::kTransform)) {
		// set function set to node
		MFnTransform fnX(obj);

		// find the bone
		BoneIT it = g_Bones.find(fnX.name().asChar());

		// if not found
		if (it == g_Bones.end()) 
		{
			// get path to bone
			MDagPath p;
			fnX.getPath(p);

			// insert entry into bone map
			std::pair<BoneIT,bool> pair_ = g_Bones.insert( std::make_pair( std::string( fnX.name().asChar() ), CBoneRef(fnX.name().asChar(),p))  );
			
			// get ref to joint
			CBoneRef& joint = pair_.first->second;

			joint.name = fnX.name().asChar();

			//////////////////////////////////////////////////////////////////////////
			
			{
				if (fnX.parentCount() != 0) 
				{
					joint.m_Parent = BuildJointRef(fnX.parent(0),doChildren);
				}
				else
				{
					joint.m_Parent = 0;
					g_ImportantTransforms.push_back(joint.id);
				}
			}

			//////////////////////////////////////////////////////////////////////////
			
			if(doChildren)
			{
				for(int i=0;i!=fnX.childCount();++i)
				{
					MObject oChild = fnX.child(i);
					if (oChild.hasFn(MFn::kTransform)) {
						unsigned int jref = BuildJointRef(oChild);
						if(jref!=0)
							joint.m_Children.push_back( jref );
					}
					else
					if ( oChild.hasFn(MFn::kMesh) || 
						 oChild.hasFn(MFn::kLight) ) 
					{
						MFnDependencyNode fnDep(oChild);
						unsigned int jref = GetObjectID(fnDep.name().asChar());
						AddObjectToBone(jref,joint.id);
					}
				}
			}
			
			//////////////////////////////////////////////////////////////////////////

			return joint.id;
		}
		return it->second.id;
	}
	return 0;
}

void OutputTransform(std::ofstream& ofs,CBoneRef& bone)
{
	double Scale[3];
	MEulerRotation EulerRotation;
	MQuaternion JO(0,0,0,1);
	MQuaternion RO(0,0,0,1);
	MQuaternion R(0,0,0,1);

	double x,y,z,w;
	// set function set to node
	MFnTransform fnX(bone.path);

	MMatrix mat = fnX.transformation().asMatrix();
	MMatrix inverse = mat.inverse();

	fnX.getScale(Scale);
	fnX.getRotation (R);

	if( fnX.object().hasFn(MFn::kJoint) )
	{
		MFnIkJoint fnJ(fnX.object());
		fnJ.getOrientation(JO);
	}

	RO = fnX.rotateOrientation(MSpace::kObject);

	MVector trans = fnX.translation(MSpace::kTransform );

	ofs << "BONE " << bone.name << std::endl;
	ofs << "\tid " << bone.id << std::endl;
	ofs << "\tparent " << bone.m_Parent << std::endl;
	ofs << "\tChildren " << bone.m_Children.size() << "\t";
	std::vector<unsigned int>::iterator it = bone.m_Children.begin();
	for( ; it != bone.m_Children.end(); ++it )
	{
		ofs << *it << " ";
	}
	ofs << std::endl;	
	ofs << "\tParentedNodes " << bone.m_ChildObjects.size() << "\t";
	it = bone.m_ChildObjects.begin();
	for( ; it != bone.m_ChildObjects.end(); ++it )
	{
		ofs << *it << " ";
	}
	ofs << std::endl;

	ofs << "\tScale\t" << Scale[0] << " " << Scale[1] << " " << Scale[2] << std::endl;
	ofs << "\tTranslation\t" << trans.x << " " << trans.y << " " << trans.z << std::endl;
	ofs << "\tRotationOrient\t" << RO.x << " " << RO.y << " " << RO.z << " " << RO.w << std::endl;
	ofs << "\tRotation\t" << R.x << " " << R.y << " " << R.z << " " << R.w << std::endl;
	ofs << "\tJointOrient\t" << JO.x << " " << JO.y << " " << JO.z << " " << JO.w << std::endl;
}


void OutputTransforms(std::ofstream& ofs)
{
	/// a map of all useful transforms
	BoneIT it = g_Bones.begin();
	for( ; it != g_Bones.end(); ++it )
	{
		std::cout << "outputting xform: " << it->first << std::endl;
		OutputTransform(ofs,it->second);
	}


	/// a map of all useful transforms
	it = g_Bones.begin();
	for( ; it != g_Bones.end(); ++it )
	{
		MFnTransform fnX(it->second.path);
		ofs << "BONEANIM " << it->second.id << std::endl;
		std::cout << "outputting bone anim " << it->first << std::endl;
		
		int StartTime = MAnimControl::minTime().as(MTime::kPALFrame);
		int EndTime   = MAnimControl::maxTime().as(MTime::kPALFrame);

		
		ofs << "\tstart " << StartTime << std::endl;
		ofs << "\tend "   << EndTime   << std::endl;

		for(int i=StartTime;i<=EndTime;++i)
		{
			MAnimControl::setCurrentTime(MTime(i,MTime::kPALFrame));
			MQuaternion quat;
			fnX.getRotation(quat);
			ofs << "\t" << quat.x << " " << quat.y << " " << quat.z << " " << quat.w << std::endl;
		}
	}


}










/// \brief	This struct is used as a 2byte identifier just before an image
/// \note	sizeof(ImageHeader) == 2
/// 
struct ImageHeader
{
	/// the width of the image as the power of two
	unsigned char m_WidthPower:4;
	/// the height of the image as a power of two
	unsigned char m_HeightPower:4;

	/// the bits per pixel
	/// 0 = 8bit
	/// 1 = 16bit
	/// 2 = 24bit
	/// 3 = 32bit
	/// 
	unsigned char m_Bpp:2;

	/// 6bits for a rainy day...
	unsigned char m_Unused:6;

	ImageHeader() : m_WidthPower(0), m_HeightPower(0), m_Bpp(0), m_Unused(0) {}
};

/// \brief	This function is another shady work around. The one problem with MImage is
///			that it always converts image data to RGBA. This is a bit of a pain because 
///			Ideally we would like a greyscale / 24bit and 32bit options. You may be able 
///			to hack around the problem with naming conventions, however I'm being very 
///			lazy and am checking every alpha value to see if it is the same as the first.
///			If they are all the same then it has a very boring alpha channel that I don't
///			want in my game enigne :p
///
bool Is24Bit(const unsigned char* px,unsigned short w, unsigned short h)
{
	unsigned char first = px[3];
	for(unsigned int i=7;i<=w*h*4;i+=4)
	{
		if(first != px[i])
		{
			return false;
		}
	}
	return true;
}

/// \brief	Is i a power of two var....
/// 
bool IsPowerOfTwo(unsigned int i)
{
	if( i == 2   || i == 4   ||
		i == 8   || i == 16  ||
		i == 32  || i == 64  ||
		i == 128 || i == 256 ||
		i == 512 || i == 1024 ) 
	{
			return true;
	}
	return false;
}

/// \brief	Takes the value v and returns the power
/// 
unsigned char LogIt(unsigned short v)
{
	switch(v) {
	case 1:
		return 0;
	case 2:
		return 1;
	case 4:
		return 2;
	case 8:
		return 3;
	case 16:
		return 4;
	case 32:
		return 5;
	case 64:
		return 6;
	case 128:
		return 7;
	case 256:
		return 8;
	case 512:
		return 9;
	case 1024:
		return 10;
	case 2048:
		return 11;
	case 4096:
		return 12;
	case 8192:
		return 13;
	case 16384:
		return 14;
	case 32768:
		return 15;
	default:
		break;
	}
	return 0;
}

// this will hold the name of the kFileTexture node and an ID number we will
// create to reference the textures from the model file.
//
std::map<std::string,unsigned int> g_MaterialToID_Map; 
unsigned int mat_id_gen = 0;


unsigned int GetMaterialID(MString &material) {
	std::map<std::string,unsigned int>::iterator it = g_MaterialToID_Map.find(material.asChar());

	if (it != g_MaterialToID_Map.end()) {
		return it->second;
	}
	return 0;
}


// if the texture is not of power 2, then it will be resized to this
// default resolution.
// 
const unsigned int g_DefaultTextureRes = 128;


/// \brief	Function to save a texture to the specified file pointer
/// \param	fp	-	the filoe pointer 
///	\param	img	-	reference to the MImage class with the loaded data
///
void OutputTexture(FILE *fp,MImage &img)
{
	ImageHeader header;
	unsigned int w,h;

	// get image size
	img.getSize(w,h);

	// check dimesnions
	if (!IsPowerOfTwo(w) || !IsPowerOfTwo(h) ) 
	{
		// oh dear, resize the image 
		w = h = g_DefaultTextureRes;
		img.resize(w,h);

		std::cerr << "Warning image not power of 2" << std::endl;
	}

	// store width and height in the header
	header.m_WidthPower  = LogIt(w);
	header.m_HeightPower = LogIt(h);

	// store pixel depth in header
	if (Is24Bit(img.pixels(),w,h)) {
		header.m_Bpp		 = 2;
	}
	else {
		header.m_Bpp		 = 3;
	}

	// write header to the file
	fwrite(&header,sizeof(ImageHeader),1,fp);

	// write the 32 bit data to the file
	if (header.m_Bpp == 3) {
		fwrite(img.pixels(),w*h*4,1,fp);
	}
	else
	{
		// write 24bit data in chunks
		for(unsigned int i=0;i<=w*h*4;i+=4)	{
			fwrite(img.pixels() + i,3,1,fp);
		}
	}
}


// this will hold the name of the kFileTexture node and an ID number we will
// create to reference the textures from the model file.
//
std::map<std::string,unsigned int> g_TextureToID_Map; 

unsigned int GetTextureID(MString &ftn) 
{
	/// try and find the texture
	std::map<std::string,unsigned int>::iterator it = g_TextureToID_Map.find(ftn.asChar());

	// if found return ID
	if (it != g_TextureToID_Map.end()) {
		return it->second;
	}

	// return 0
	return 0;
}

// global used to generate texture ID numbers
unsigned int id_gen=1;

/// \brief	This function processes all MFn::kFile nodes in the scene. 
///			It basically writes the raw data out to the file and creates
///			a set of reference ids for the textures
/// \param	The path to the output binary file
/// 
void DoTextures(std::string outfile)
{
	// vector to hold the texture filenames
	MObjectArray FileNodes;

	// create an iterator for the texture files
	MItDependencyNodes itDep(MFn::kFileTexture);

	// set the id generator to zero
	id_gen = 1;

	// loop through fileTexture nodes until done
	while (!itDep.isDone()) 
	{
		// create a function set to get some useful attributes
		MFnDependencyNode fnDep(itDep.item());

		// store the MObject to the file texture
		FileNodes.append(itDep.item());

		// insert the name of the node and the ID number into the map for lookup when
		// exporting materials.
		g_TextureToID_Map.insert( std::make_pair( fnDep.name().asChar(), id_gen++) );

		// move to next texture node
		//
		itDep.next();
	}

	// open the output file
	FILE* fp = fopen(outfile.c_str(),"wb");

	// if OK
	if (fp) {

		// the first 2 bytes of the file is a number identifying how many textures
		// are in this file. 
		unsigned short texCount = FileNodes.length();
		fwrite(&texCount,sizeof(unsigned short),1,fp);

		// iterate through all FileNodes
		for( int i=0 ; i != FileNodes.length(); ++i )
		{
			MImage img;
		
			// attempt to read the image file
			if( MS::kSuccess == img.readFromTextureNode( FileNodes[i] ) )
			{
				// output the texture if sucessful
				OutputTexture(fp,img);
				img.release();
			}
			else
			{
				// output warning and 2byte field of zeros if not opened
				unsigned short temp = 0x0000;
				fwrite(&temp,sizeof(unsigned short),1,fp);
			}

		}

		// close the file when done.
		fclose(fp);
	}

}

void OutputMesh(std::ofstream& ofs,MObject &obj)
{
	// attach a function set to the mesh node. We access all of the meshes data 
	// through the function set
	// 
	MFnMesh fnMesh(obj);


	// Maya has a tendancy of storing additional nodes and objects all over the place.
	// We may come accross meshes that are nodes used purely as history or other 
	// construction data. In our case, we want to ignore all 'intermediate objects'
	//
	if ( !fnMesh.isIntermediateObject() ) 
	{
		unsigned int iParent = BuildJointRef( fnMesh.parent(0) );
		unsigned int iNode =  GetObjectID(fnMesh.name().asChar());

		AddChildToBone(iNode,iParent);

		// write out some info about the mesh
		// 
		ofs << "MESH "
			<< fnMesh.name().asChar() 
			<< "\n\tID= "
			<< GetObjectID(fnMesh.name().asChar())
			<< "\n\tParentID= "
			<< iParent
			<< "\n\tVertexCount= "
			<< fnMesh.numVertices() 
			<< "\n\tNormalCount= "
			<< fnMesh.numNormals()
			<< "\n\tTexCoordCount= "
			<< fnMesh.numUVs()
			<< "\n\tFaceCount= "
			<< fnMesh.numPolygons()
			<< std::endl;

		int i,j;

		// this will hold the returned vertex positions
		MPointArray vertices;
	
		// use the function set to get the points 
		fnMesh.getPoints(vertices);



		// go through each vertex in the array and write it to the file 
		//
		ofs << "\n\tVertices\n";
		for(i=0;i!=vertices.length() ; ++i)
		{					
			// output 
			//
			ofs << "\t\t" << vertices[i].x << " " << vertices[i].y << " " << vertices[i].z << "\n";
		}



		// this will store the normals
		MFloatVectorArray normals;

		// get the normals
		fnMesh.getNormals(normals);

		// go through each normal in the array and write it to the file 
		//
		ofs << "\n\tNormals\n";
		for(i=0;i!=normals.length() ; ++i)
		{
			// output the normal 
			// 
			ofs << "\t\t" << normals[i].x << " " << normals[i].y << " " << normals[i].z << "\n";
		}

		// these two arrays will hold the u & v texture co-ordinates
		//
		MFloatArray u_coords;
		MFloatArray v_coords;

		// get the uv coordinates
		//
		ofs << "\n\tTexCoords\n";
		fnMesh.getUVs(u_coords,v_coords);

		// go through each uv in the array and write it to the file 
		//
		for(i=0;i!=u_coords.length() ; ++i)
		{
			ofs << "\t\t" << u_coords[i] << " " << v_coords[i] << "\n";
		}



		MObjectArray Shaders;
		MIntArray    FaceIndices;

		int k = 0;

		fnMesh.getConnectedShaders(0,Shaders,FaceIndices);

	
		std::vector< unsigned int > ShaderIDs;

		ShaderIDs.push_back(-1);

		for(k=0;k!=Shaders.length();++k)
		{
			MFnDependencyNode fnShader(Shaders[k]);

			MPlug sshader = fnShader.findPlug("surfaceShader");

			MPlugArray materials;

			sshader.connectedTo(materials,true,true);

			for(unsigned int l=0;l<materials.length();++l)
			{
				switch(materials[l].node().apiType()) {
				case MFn::kPhong:
					{
						MFnDependencyNode fnMat(materials[l].node());
						ShaderIDs.push_back( GetMaterialID(fnMat.name()) );
						l = materials.length();
					}
					break;
				case MFn::kLambert:
					break;
				default:
					break;
				}
			}
		
		}



		k = 0;


		// to get at the face data, we need to use a mesh polygon iterator.
		//
		MItMeshPolygon itPoly(fnMesh.object());

		// keep looping until the iterator has finished
		// 
		while ( !itPoly.isDone() )
		{

			// get the number of vertices in this face
			unsigned int vcount = itPoly.polygonVertexCount();
			int matInd;
				{
					matInd =  ShaderIDs[ FaceIndices[k]+1 ];
				}
			// output some info about the face
			//
			ofs << "\n\tFACE "
				<< vcount 
				<< "\n\t\tMat= "
				<< matInd
				<< std::endl;

			// for each vertex on the face....
			//
			for(i = 0; i!= vcount; ++i)
			{
				// have to get the uv index seperately
				int uv_index;
				itPoly.getUVIndex(i,uv_index);


				// output indices for this vertex
				ofs << "\t\t" << itPoly.vertexIndex(i) << " " << itPoly.normalIndex(i) << " " << uv_index << "\n";
			}

			++k;

			// move to next poly in the mesh
			//
			itPoly.next();
		}
	}
	ofs << "\n";
}

void OutputLight(std::ofstream& ofs,MObject &obj)
{
	MFnLight fnLight(obj);

	if ( !fnLight.isIntermediateObject() ) 
	{
		unsigned int iParent;
		if (fnLight.parentCount()>0) {
			iParent = BuildJointRef( fnLight.parent(0) );
		}
		unsigned int iNode =  GetObjectID(fnLight.name().asChar());

		AddChildToBone(iNode,iParent);

		// get light color
		MColor color(0.0f,0.0f,0.0f);
		color = fnLight.color();

		// output some light info
		ofs << "LIGHT "

			// output name
			<< fnLight.name().asChar()
			<< "\n\tid= "
			<< iNode
			
			// output light type
			<< "\n\tType=\t\t" 
			<< obj.apiTypeStr()

			// output color 
			<< "\n\tColor=\t\t"
			<< color.r
			<< " "
			<< color.g
			<< " "
			<< color.b 

			// output intensity
			<< "\n\tIntensity=\t" 
			<< fnLight.intensity()

			// output light position
			<< "\n\tParent=\t"
			<< iParent
			<< std::endl;


		// if you need to get specific attributes associated with specific
		// light types then use something like this....
		//
		switch(obj.apiType()) 
		{
		case MFn::kPointLight:
			{
				MFnPointLight fnPoint(obj);

			}
			break;
		case MFn::kAmbientLight:
			{
				MFnAmbientLight fnAmbient(obj);

			}
			break;
		case MFn::kSpotLight:
			{
				MFnSpotLight fnSpot(obj);

			}
			break;
		case MFn::kDirectionalLight:
			{
				MFnDirectionalLight fnDir(obj);

			}
			break;
		case MFn::kAreaLight:
			{
				MFnAreaLight fnArea(obj);

			}
			break;
		default:
			break;
		}
	}

}

/// \brief	This function outputs the material color via manually getting a plug to the attribute
/// \param	ofs		-	the output file stream
/// \param	fn		-	the function set
/// \param	name	-	name of the attribute
/// 
bool OutputMaterialColor(std::ostream& ofs,MFnDependencyNode& fn,std::string name)
{
	MPlug p;// = fn.findPlug();
	

	// if it is valid
//	if(!p.isNull())
	{
		std::string r = name;
		r += "R";
		std::string g = name;
		g += "G";
		std::string b = name;
		b += "B";
		std::string a = name;
		a += "A";

		// get the color value
		MColor color;
	
		// get a plug to the attribute
		p = fn.findPlug(r.c_str());
		p.getValue(color.r);
		p = fn.findPlug(g.c_str());
		p.getValue(color.g);
		p = fn.findPlug(b.c_str());
		p.getValue(color.b);
		p = fn.findPlug(a.c_str());
		p.getValue(color.a);

		p = fn.findPlug(name.c_str());

		// will hold the txture node name
		MString texname;

		// get plugs connected to colour attribute
		MPlugArray plugs;
		p.connectedTo(plugs,true,true);

		// see if any file textures are present
		for(int i=0;i!=plugs.length();++i)
		{
			// if file texture found
			if (plugs[i].node().apiType() == MFn::kFileTexture) 
			{
				// bind a function set to it ....
				MFnDependencyNode fnDep(plugs[i].node());

				// to get the node name
				texname = fnDep.name();

				// stop looping
				break;
			}
		}

		// output the name, color and texture ID
		ofs << "\t" << name << " "
			<< color.r << " "
			<< color.g << " "
			<< color.b << " "
			<< color.a << " tex= "
			<< GetTextureID( texname ) << "\n";

		return true;
	}
	return false;    
}



int main(int argc,char** argv)
{


	// if the argument count is one, then no files were specified
	//
	if (argc==1) {
		std::cerr << "[ USAGE ] Exporter <infile.mb> { <outfile.ext> }" << std::endl;
		return 0;
	}

	std::string outFilePrefix;



	// var to hold the output file name
	std::string outfilename;

	// if only one argument was specified, I'll assume that it is a maya binary
	// or ascii file, in which case use this - 'mb' or 'ma' and add a 'mod' extension
	// to the end of the filename 
	// 
	outFilePrefix = argv[1];
	outFilePrefix.resize(outFilePrefix.size()-2);
	
	if (argc == 2 )
	{

		outfilename = outFilePrefix;
		outfilename += "mod";
	}
	else
	{
		// if outfile file was specified, use that
		//
		outfilename = argv[2];
	}

	outFilePrefix += "tex";


	// initialise the maya library - This basically starts up Maya
	// 
	MLibrary::initialize(argv[0]);




	// open the file that the user specified 
	//
	if( MS::kSuccess == MFileIO::open(argv[1],NULL,true) )
	{

		// do the texture file for a laugh....
		//
		DoTextures(outFilePrefix);

		// create an output file stream
		//
		std::ofstream ofs(outfilename.c_str());

		// make sure it opened OK
		//
		if (!ofs) {
			std::cerr << "Could not open output file " << outfilename << std::endl;
			MLibrary::cleanup();
			return 1;
		}

		// do the material data first....
		//
		{
			// iterate through the mesh nodes in the scene
			//
			MItDependencyNodes itDep(MFn::kInvalid);

			// we have to keep iterating until we get through all of the nodes in the scene
			//
			while (!itDep.isDone()) 
			{

				switch(itDep.item().apiType()) {

				// if found phong shader
				case MFn::kPhong:
					{
						MFnPhongShader fnPhong( itDep.item() );

						g_MaterialToID_Map.insert( std::make_pair( fnPhong.name().asChar(), mat_id_gen++ ) );

						ofs << "PHONG " << fnPhong.name().asChar() << "\n";
						OutputMaterialColor(ofs,fnPhong,"ambientColor");
						OutputMaterialColor(ofs,fnPhong,"color");
						OutputMaterialColor(ofs,fnPhong,"specularColor");
						OutputMaterialColor(ofs,fnPhong,"incandescence");
						OutputMaterialColor(ofs,fnPhong,"transparency");
						ofs	<< "\tshininess " << (fnPhong.cosPower()) << std::endl; 

					}
					break;

				// if found lambert shader
				case MFn::kLambert:
					{
						MFnLambertShader fnLambert( itDep.item() );

						g_MaterialToID_Map.insert( std::make_pair( fnLambert.name().asChar(), mat_id_gen++ ) );

						ofs << "LAMBERT " << fnLambert.name().asChar() << "\n";
						OutputMaterialColor(ofs,fnLambert,"ambientColor");
						OutputMaterialColor(ofs,fnLambert,"color");
						OutputMaterialColor(ofs,fnLambert,"incandescence");
						OutputMaterialColor(ofs,fnLambert,"transparency");
					}
					break;

				// if found blinn shader
				case MFn::kBlinn:
					{
						MFnBlinnShader fnBlinn( itDep.item() );

						g_MaterialToID_Map.insert( std::make_pair( fnBlinn.name().asChar(), mat_id_gen++ ) );

						ofs << "BLINN " << fnBlinn.name().asChar() << "\n";
						OutputMaterialColor(ofs,fnBlinn,"ambientColor");
						OutputMaterialColor(ofs,fnBlinn,"color");
						OutputMaterialColor(ofs,fnBlinn,"specularColor");
						OutputMaterialColor(ofs,fnBlinn,"incandescence");
						OutputMaterialColor(ofs,fnBlinn,"transparency");
						ofs	<< "\teccentricity" << (fnBlinn.eccentricity()) << std::endl; 
						ofs	<< "\tspecularRollOff" << (fnBlinn.specularRollOff()) << std::endl; 
					}
					break;
				default:
					break;
				}

				itDep.next();
			}
		}


		{
			// iterate through the mesh nodes in the scene
			//
			MItDependencyNodes itDep(MFn::kInvalid);

			// we have to keep iterating until we get through all of the nodes in the scene
			//
			while (!itDep.isDone()) 
			{
				// 
				MObject item = itDep.item();

				switch(item.apiType()) 
				{
				case MFn::kMesh:
					OutputMesh(ofs,item);
					break;

				case MFn::kLight:
				case MFn::kAreaLight:
				case MFn::kAmbientLight:
				case MFn::kDirectionalLight:
				case MFn::kPointLight:
				case MFn::kSpotLight:
					OutputLight(ofs,item);
					break;

				default:
					break;
				}


				// move to next node
				itDep.next();
			}
		}

		OutputTransforms(ofs);


		// close the output file
		ofs.close();
	}
	else
	{
		std::cerr << "Could not open file " << argv[1] << std::endl;
	}


	// cleanup maya data

	MLibrary::cleanup();

	return 0;
}