/*----------------------------------------------------------------------------------*\
 *	file:	Main.c
 *	author:	
 *	date:	
 *	notes:	
\*----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------*\
 *		Includes
\*----------------------------------------------------------------------------------*/

/* c std library includes */
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* windows compilation requires windows.h */
#ifdef WIN32
	/* This speeds up compile time on WIN32 by excluding lots of rubbish stuff */
	#define WIN32_LEAN_AND_MEAN	
	#include <Windows.h>
#endif

/* open gl includes */
#include <GL/glut.h>

/* Add our includes */
#include "Vector.h"
#include "Matrix.h"
#include "ModelLoader.h"
#include "TextureLoader.h"
#include "font.h"
#include "NURBS.h"

/*----------------------------------------------------------------------------------*\
 *		Macros & Constants
\*----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------*\
 *		Data Types
\*----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------*\
 *		Globals
\*----------------------------------------------------------------------------------*/
int winw, winh;

int wire=0;

float roll=0;


float blx=0,bly=1,blz=4;
float tlx=0,tly=0,tlz=1;
float trx=0,try=0,trz=1;
float brx=0,bry=0,brz=1,brrx=0, brry=0;

/*----------------------------------------------------------------------------------*\
 *		Function Prototypes
\*----------------------------------------------------------------------------------*/

void draw_tools()
{

	glColor3f(0.5,0.5,0.5);
	glBegin (GL_QUADS);
	glVertex2i (0,0);
	glVertex2i (0,20);
	glVertex2i (100,20);
	glVertex2i (100,0);
	glEnd();

	glColor3f(0.8,0.8,0.8);
	glBegin (GL_LINE_LOOP);
	glVertex2i (0,0);
	glVertex2i (0,20);
	glVertex2i (100,20);
	glVertex2i (100,0);
	glEnd();

	glColor3f(0.2,0.3,0.5);
	
	FontDraw(GLUT_BITMAP_HELVETICA_18,"load me!", 5,19);

}

/*----------------------------------------------------------------------------------*\
 *		Functions
\*----------------------------------------------------------------------------------*/
void draw ()
{
	/*
	 *	Clear the screen
	 */
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	/*
	 *	Set up the viewport and projection transformation for the toolbar
	 */
	glViewport(0,0,100,winh);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,100,winh,0,-1,1);
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);

	/*
	 *	Draw the toolbar
	 */
	draw_tools();

	/*
	 *	Set up the transform for the 3D view
	 */
	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);

	/*
	 *	Set wireframe mode
	 */
	if(wire==1)
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	else
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

	//===========================================================top left-top

	glViewport(100,winh/2,(winw-100)/2,winh/2);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	glOrtho( tlz*(-1+tlx), tlz*(1+tlx),
		     tlz*(-1+tly), tlz*(1+tly),
			 -100,100);
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
	glRotatef(90,1,0,0);
	
	glColor3f(1,1,1);

	/*
	 *	Do camera movement
	 */
	glRotatef(roll,0,1,0);

	/*
	 *	Draw everything currently loaded
	 */
	drawall();

	//====================================================bottom left-front

		glViewport(100,0,(winw-100)/2,winh/2);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	glOrtho( blz*(-1+blx), blz*(1+blx),
		     blz*(-1+bly), blz*(1+bly),
			 -100,100);
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
	glRotatef(90,0,1,0);

glColor3f(1,1,1);

	/*
	 *	Draw everything currently loaded
	 */
	drawall();


	//=======================================================bottom right- persp
		glViewport(100+(winw-100)/2,0,(winw-100)/2,winh/2);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45,4/3,1,100);
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
	gluLookAt(0,0,3,
			  0,0,0,
			  0,1,0);

	glTranslatef(0,0,brz);
	glRotatef(brrx,1,0,0);
	glRotatef(brry,0,1,0);
	glTranslatef(brx,bry,0);

	glColor3f(1,1,1);


	/*
	 *	Draw everything currently loaded
	 */
	drawall();

	//====================================================================
	//top right-side

		glViewport(100+(winw-100)/2,winh/2,(winw-100)/2,winh/2);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho( trz*(-1+trx), trz*(1+trx),
		     trz*(-1+try), trz*(1+try),
			 -100,100);
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
	glColor3f(1,1,1);


	/*
	 *	Draw everything currently loaded
	 */
	drawall();

	

	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

	drawAllNURB();

	glutSwapBuffers();
}

void resize (int x, int y)
{	
	/*
	 *	Store window width and hieght of window globally
	 */
	winw = x;
	winh = y;
}	

enum VIEW 
{
	VIEW_NONE,
	TOOLBAR,
	TOP_LEFT,
	BOTTOM_LEFT,
	TOP_RIGHT,
	BOTTOM_RIGHT
};
typedef enum VIEW VIEW;

VIEW view=0;
int lastx,lasty;
int left=0,right=0,middle=0;
int AltPressed = 0;

void Mouse(int button,int state,int x,int y)
{
	lastx = x;
	lasty = y;

	if(glutGetModifiers() & GLUT_ACTIVE_ALT)
	{
		AltPressed=1;
	}
	else
		AltPressed=0;

	if(state == GLUT_DOWN)
	{
		switch(button) {
		case GLUT_LEFT_BUTTON:
			if(middle==1)
			{
				glutSetCursor(GLUT_CURSOR_LEFT_RIGHT);
			}
			else
			{
				glutSetCursor(GLUT_CURSOR_CYCLE);
			}
			left=1;
			break;
		case GLUT_RIGHT_BUTTON:
			right=1;
			break;
		default:
			if(left==1)
			{
				glutSetCursor(GLUT_CURSOR_LEFT_RIGHT);
			}
			else{
				glutSetCursor(GLUT_CURSOR_SPRAY);
			}
			middle=1;
			break;
		}
		if(x<100)
		{
			if (y<20) {
				char buffer[256];
				printf ("fuck you!!!\n");
				scanf("%s",buffer);
				loadmod(buffer);
			}
			else
			if (y<40) {
				if (wire == 0) 
					wire = 1;
				else
					wire=0;
			}
		}
		else
		{
			if (y<winh/2) {
				if (x<(winw-100)/2+100) {
					// tl
					if(button==GLUT_LEFT_BUTTON && middle == 0)
					{
						glutSetCursor(GLUT_CURSOR_DESTROY);
					}
					view = TOP_LEFT;
				}
				else
				{
					// tr
					view = TOP_RIGHT;
				}
			}
			else
			{
				if (x<(winw-100)/2+100) {
					// bl
					view = BOTTOM_LEFT;
				}
				else
				{
					// br
					view = BOTTOM_RIGHT;
				}
			}
		}
	}
	else
	{
		
		switch(button) {
		case GLUT_LEFT_BUTTON:
			left=0;
			break;
		case GLUT_RIGHT_BUTTON:
			right=0;
			break;
		default:
			middle=0;
			break;
		}

		glutSetCursor(GLUT_CURSOR_INHERIT);
	}
	glutPostRedisplay ();
}


void Motion(int x,int y)
{
	int dx = x-lastx;
	int dy = y-lasty;
	lastx = x;
	lasty = y;


	if (AltPressed) {

		switch(view)
		{
		case BOTTOM_RIGHT:
			if(left==1 && middle==1)
			{
				brz-=(0.01f*dx);
			}
			else
			if(middle==1)
			{
				brx+=(0.01f*dx);
				bry-=(0.01f*dy);
			}
			else
			if(left==1)
			{
				brrx+=(dy);
				brry+=(dx);
			}
			break;
		case TOP_RIGHT:
			if(left==1 && middle==1)
			{
				trz-=(0.01f*dx);
			}
			else
			if(middle==1)
			{
				trx-=(0.01f*dx);
				try+=(0.01f*dy);
			}
			break;
			
		case BOTTOM_LEFT:
			if(left==1 && middle==1)
			{
				blz-=(0.01f*dx);
			}
			else
			if(middle==1)
			{
				blx-=(0.01f*dx);
				bly+=(0.01f*dy);
			}
			break;
		case TOP_LEFT:
			if(left==1 && middle==1)
			{
				tlz-=(0.01f*dx);
			}
			else
			if(middle==1)
			{
				tlx-=(0.01f*dx);
				tly+=(0.01f*dy);
			}
			break;
		default:
			break;
		}

	}
	else
	{
		// do selection
	}
	glutPostRedisplay ();
}

void Keyboard(unsigned char key,int x,int y)
{
	if (key == '4') {
		wire = 1;
	}
	if (key == '5') {
		wire = 0;
	}
	
	glutPostRedisplay ();
}
void init()
{
	float fogColor[4] = {0.5, 0.5, 0.5, 1.0};
	float ambient[4] = {0.3, 0.3, 0.3, 1.0};
	float diffuse[4] = {0.8, 0.8, 0.8, 1.0};
	float specular[4] = {1.0, 1.0, 1.0, 1.0};
	float glow[4] = {0, 0, 0, 0};

	glClearColor (0.5,0.5,0.5,0.0);

	glShadeModel(GL_SMOOTH);

	glFogi (GL_FOG_MODE, GL_LINEAR);
	glFogfv(GL_FOG_COLOR, fogColor);
	glFogf (GL_FOG_DENSITY, 0.35);
	glFogf (GL_FOG_START, 4.0);
	glFogf (GL_FOG_END, 5.5);

	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);

	glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT, GL_EMISSION, glow);
	glMaterialf(GL_FRONT, GL_SHININESS, 30);

	glCullFace(GL_BACK);

	loadmod("girl.obj");

//	glEnable(GL_CULL_FACE);
//	glEnable(GL_FOG);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

}

/*----------------------------------------------------------------------------------*\
 *		Main
\*----------------------------------------------------------------------------------*/

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE|GLUT_DEPTH);
	glutInitWindowSize(500,500);
	glutCreateWindow("sam");


	init();

	initNurb();

	glutDisplayFunc(draw);
	glutReshapeFunc(resize);
	glutMouseFunc(Mouse);
	glutMotionFunc(Motion);
	glutKeyboardFunc(Keyboard);

	glutMainLoop();
	return 0;
}







