
/* c std library includes */
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* windows compilation requires windows.h */
#ifdef WIN32
	/* This speeds up compile time on WIN32 by excluding lots of rubbish stuff */
	#define WIN32_LEAN_AND_MEAN	
	#include <Windows.h>
#endif

/* open gl includes */
#include <GL/gl.h>
#include <GL/GLU.h>

/*
 *	The Nurbs Surface Renderer
 */
GLUnurbs *TheSurface;

struct NURB
{

	float* ControlPoints; 


	/*
	*	The number of control points in the surface directions
	*/
	int NumCVsU; 
	int NumCVsV; 

	/*
	*	The knot vectors in the two surface directions	
	*/
	float* KnotsU;
	float* KnotsV;

	/*
	*	The number of knots in both surface directions
	*/
	int NumKnotsU;
	int NumKnotsV;

	/*
	*	The degree of the surface in the two surface directions
	*/
	int DegreeU;
	int DegreeV;

	int id;

	struct NURB* next;

};

typedef struct NURB NURB;

NURB* pNURB = NULL;

extern int gid;

void createNURB ()
{
	NURB* p = (NURB*)malloc(sizeof(NURB));
	memset(p,0,(sizeof(NURB)));

	p->id = ++gid;

	p->next = pNURB;
	pNURB = p;
}

void deleteNURB(int id)
{
	NURB* previous=NULL;
	NURB* current=pNURB;

	while (current!=NULL)
	{
		if(current->id == id)
		{
			if(previous!=NULL)
				previous->next=current->next;
			else
				pNURB=current->next;
			free (current);
			return;
		}
		previous=current;
		current=current->next;
	}
}

void drawNURB(int id)
{
}

void drawAllNURB()
{
	NURB* p = pNURB;
	while (p!=NULL)
	{
		gluBeginSurface(TheSurface);

		/*
		*	This actually draws the surface
		*/
		gluNurbsSurface(TheSurface,
				p->NumKnotsU, p->KnotsU, 
				p->NumKnotsV, p->KnotsV,
				p->NumCVsU * 3, 
				3, 
				p->ControlPoints,
				p->DegreeU + 1, 
				p->DegreeV + 1, 
				GL_MAP2_VERTEX_3);

		/*
		*	Finish drawing the surface
		*/
		gluEndSurface(TheSurface);
		p = p->next;
	}

}
void initNurb()
{
	glEnable(GL_AUTO_NORMAL);
	glEnable(GL_NORMALIZE);

	/*
	 *	Create a new nurbs surface
	 */
	TheSurface = gluNewNurbsRenderer();

	/*
	 *	Set display mode and sampling tolerance
	 */
	gluNurbsProperty(TheSurface, GLU_SAMPLING_TOLERANCE, 60.0);
	gluNurbsProperty(TheSurface, GLU_DISPLAY_MODE, GLU_OUTLINE_POLYGON);
}

void loadNURB(char* fileName)
{
	FILE* fp=fopen(fileName, "r");
	char buffer[256];
	int surf,i,j,k;
	fscanf (fp,"%s%d",buffer,&surf);
	for (i=0;i<surf;i++)
	{
		createNURB();
		fscanf (fp,"%s%s%s",buffer,buffer,buffer);
		fscanf (fp,"%d%s%d",&pNURB->DegreeU,buffer,&pNURB->DegreeV);
		fscanf (fp,"%s%d%s",buffer,&pNURB->NumKnotsU,buffer);
		pNURB->KnotsU = (float*)malloc(sizeof(float)*(pNURB->NumKnotsU));
		for(j=0;j<pNURB->NumKnotsU;j++)
		{
			fscanf (fp,"%f",&pNURB->KnotsU[j]);
		}

		fscanf (fp,"%s%d%s",buffer,&pNURB->NumKnotsV,buffer);
		pNURB->KnotsV = (float*)malloc(sizeof(float)*(pNURB->NumKnotsV));
		for(j=0;j<pNURB->NumKnotsV;j++)
		{
			fscanf (fp,"%f",&pNURB->KnotsV[j]);
		}
		fscanf (fp,"%s%d%s%d%s",buffer,&pNURB->NumCVsU,buffer,&pNURB->NumCVsV,buffer);
		k= 3 * pNURB->NumCVsU * pNURB->NumCVsV;
		pNURB->ControlPoints = (float*)malloc(sizeof(float)*k);
		for(j=0;j<k;j++)
		{
			fscanf (fp,"%f",&pNURB->ControlPoints[j]);
		}
	}
	fclose(fp);
}

void saveNURB(char* filename)
{

}

