/*----------------------------------------------------------------------------------*\
 *	file:	Vector.h
 *	author:	
 *	date:	
 *	notes:	
\*----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------*\
 *		Sentry guards
\*----------------------------------------------------------------------------------*/
#ifndef VECTOR_H_
#define VECTOR_H_

#ifdef __cplusplus
extern "C" {
#endif

	/*------------------------------------------------------------------------------*\
	 *		Macros & Constants
	\*------------------------------------------------------------------------------*/

	/*------------------------------------------------------------------------------*\
	 *		Data Types
	\*------------------------------------------------------------------------------*/

	/*
	 *	Define a new type called 'vec' that represents a 3D vector
	 */
	struct vec 
	{
		union {
			struct {
				float x;
				float y;
				float z;
			};
			float data[3];
		};
	};

	/*
	 *	At the moment our data type is called 'struct vec'. To make our 
	 *	lives easier we rename that to just vec by using the typedef.
	 */
	typedef struct vec vec;

	/*------------------------------------------------------------------------------*\
	 *		Externs
	\*------------------------------------------------------------------------------*/

	/*
	 *	The function 'dotproduct' is defined within Vector.c. If we want to be able
	 *	to use that function in other *.c files we have to provide a function
	 *	prototype within a header file and declare it as 'extern' (ie, this function 
	 *	is declared externally).
	 *
	 *	\brief	This function performs the dot product between 2 vectors.
	 *	\param	a	-	the first of 2 vectors
	 *	\param	b	-	the second of two vectors
	 *	\return	the dot product as a float
	 */
	extern float dotproduct (vec*,vec*);


#ifdef __cplusplus
}
#endif

#endif